//GameButton.  Zhe  WordSnake-2002.

unit GameButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics;

type TGameButtonState = (rbsNeutral,rbsPressed,rbsHighlighted);

type
  TGameButton = class(TGraphicControl)
  private
    fNeutralTex:TPicture;
    fPressTex:TPicture;
    fHighlightTex:TPicture;
    fButtonState:TGameButtonState;
    fMouseIn:Boolean;
    fUseHighlight:Boolean;
    fAutosize:boolean;
    fAutoWidth:Integer;
    fAutoHeight:Integer;
    fCaption:String;
    fTextLeft:integer;
    fTextTop:integer;
    fFont:TFont;
    fFontChanged:Boolean;
    fGroupIndex:Integer;
    fTransparent:Boolean;
    procedure SetNeutralTex(value:TPicture);
    procedure SetPressTex(value:TPicture);
    procedure SetHighlightTex(value:TPicture);
    procedure MDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure MUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure HelloCursor(var Message:TWMMouse); message CM_MOUSEENTER;
    procedure GoodByeCursor(var Message:TWMMouse); message CM_MOUSELEAVE;
    procedure SetUseHighlight(value:boolean);
    procedure SetAutosize(value:boolean); reintroduce;
    procedure SetCaption(value:String);
    procedure SetTextLeft(value:integer);
    procedure SetTextTop(value:integer);
    procedure FontChanged(Sender:TObject);
    procedure SetGroupIndex(value:Integer);
    procedure SetButtonState(value:TGameButtonState);
    procedure SetTransparent(value:Boolean);
  public
    constructor Create(AOwner:TComponent);override;
    destructor Destroy;override;
    procedure Paint; override;
    property ButtonState:TGameButtonState read FButtonState write SetButtonState;
  published
    property NeutralTexture:TPicture read fNeutralTex write SetNeutralTex;
    property PressTexture:TPicture read fPressTex write SetPressTex;
    property HighlightTexture:TPicture read fHighlightTex write SetHighlightTex;
    property UseHighlight:boolean read fUseHighlight write SetUseHighlight default false;
    property Autosize:boolean read fAutosize write SetAutosize default false;
    property Caption:String read fCaption write SetCaption;
    property TextLeft:integer read fTextLeft write SetTextLeft default 0;
    property TextTop:integer read fTextTop write SetTextTop default 0;
    property Font:TFont read fFont write fFont;
    property GroupIndex:Integer read fGroupIndex write SetGroupIndex default 0;
    property Transparent:boolean read fTransparent write SetTransparent default false;
    property Enabled;
    property ShowHint;
    property ParentShowHint;
    property Tag;
    property Visible;
    property PopupMenu;
    property Anchors;
    property Action;
    property OnClick;
    property OnContextPopup;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Standard', [TGameButton]);
end;

constructor TGameButton.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  fNeutralTex:=TPicture.Create;
  fPressTex:=TPicture.Create;
  fHighlightTex:=TPicture.Create;
  fFont:=TFont.Create;
  Font.OnChange:=FontChanged;
  Canvas.Brush.Style:=bsClear;
  OnMouseDown:=MDown;
  OnMouseUp:=MUp;
end;

destructor TGameButton.Destroy;
begin
  fNeutralTex.Free;
  fPressTex.Free;
  fHighlightTex.Free;
  fFont.Free;
  inherited Destroy;
end;

procedure TGameButton.SetNeutralTex(value:TPicture);
begin
  fNeutralTex.Assign(value);
  if fAutoSize then
    begin
      SetAutoSize(true);
      exit;
    end;
  Paint;
end;

procedure TGameButton.SetPressTex(value:TPicture);
begin
  fPressTex.Assign(value);
  if fAutoSize then
    begin
      SetAutoSize(true);
      exit;
    end;
  Paint;
end;

procedure TGameButton.SetHighlightTex(value:TPicture);
begin
  fHighlightTex.Assign(value);
  if fAutoSize then
    begin
      SetAutoSize(true);
      exit;
    end;
  Paint;
end;

procedure TGameButton.Paint;
begin
  if (fButtonState<>rbsPressed) and fMouseIn
    then
      if fUseHighlight
        then Canvas.Draw(0,0,fHighlightTex.Graphic)
        else Canvas.Draw(0,0,fNeutralTex.Graphic)
    else
      if fButtonState=rbsPressed then Canvas.Draw(0,0,fPressTex.Graphic)
      else Canvas.Draw(0,0,fNeutralTex.Graphic);
  if fFontChanged then Canvas.Font.Assign(fFont);
  if fCaption<>''then Canvas.TextOut(TextLeft,TextTop,Caption);
end;

procedure TGameButton.MDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Index:integer;
begin
  if fGroupIndex<>0 then
    begin
      for Index:=0 to Parent.ControlCount-1 do
        if (Parent.Controls[Index] is TGameButton)
        and((Parent.Controls[Index] as TGamebutton).GroupIndex=fGroupIndex) then
          begin
            (Parent.Controls[Index] as TGameButton).fButtonState:=rbsNeutral;
            (Parent.Controls[Index] as TGameButton).Paint;
          end;
    end;
  fButtonState:=rbsPressed;
  Paint;
end;

procedure TGameButton.MUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (fGroupIndex=0) then begin
    fButtonState:=rbsNeutral;
    Paint;
  end;
end;

procedure TGameButton.HelloCursor(var Message:TWMMouse);
begin
  fMouseIn:=true;
  if GroupIndex=0 then fButtonState:=rbsHighlighted;
  Paint;
end;

procedure TGameButton.GoodByeCursor(var Message:TWMMouse);
begin
  fMouseIn:=false;
  if GroupIndex=0 then fButtonState:=rbsNeutral;
  Paint;
end;

procedure TGameButton.SetUseHighlight(value:boolean);
begin
  fUseHighlight:=value;
  if fAutoSize then SetAutoSize(true);
  Paint;
end;

procedure TGameButton.SetAutoSize(value:boolean);
begin
  fAutosize:=value;
  if value then
    begin
      if (fNeutralTex.Graphic<>nil)then
        begin
          fAutoWidth:=fNeutralTex.Graphic.Width;
          fAutoHeight:=fNeutralTex.Graphic.Height;
        end;
      if (fPressTex.Graphic<>nil)then
        begin
          if fPressTex.Graphic.Width>fAutoWidth then
             fAutoWidth:=fPressTex.Graphic.Width;
          if fPressTex.Graphic.Height>fAutoHeight then
             fAutoHeight:=fPressTex.Graphic.Height;
        end;
      if fUseHighlight and(fHighlightTex.Graphic<>nil)then
        begin
          if fHighlightTex.Graphic.Width>fAutoWidth then
             fAutoWidth:=fHighlightTex.Graphic.Width;
          if fHighlightTex.Graphic.Height>fAutoHeight then
             fAutoHeight:=fHighlightTex.Graphic.Height;
        end;
      Width:=fAutoWidth;
      Height:=fAutoHeight;
      Paint;
    end;
end;

procedure TGameButton.SetCaption(value:string);
begin
  fCaption:=value;
  Paint;
end;

procedure TGameButton.SetTextLeft(value:integer);
begin
  fTextLeft:=value;
  Paint;
end;

procedure TGameButton.SetTextTop(value:integer);
begin
  fTextTop:=value;
  Paint;
end;

procedure TGameButton.FontChanged(Sender:TObject);
begin
  fFontChanged:=true;
  Paint;
  fFontChanged:=false;
end;

procedure TGameButton.SetGroupIndex(value:integer);
begin
  fGroupIndex:=value;
  Paint;
end;

procedure TGameButton.SetButtonState(value:TGameButtonState);
begin
  fButtonState:=value;
  Paint;
end;

procedure TGameButton.SetTransparent(value:Boolean);
begin
  fTransparent:=value;
  if fNeutralTex.Graphic<>nil then
    fNeutralTex.Graphic.Transparent:=value;
  if fHighlightTex.Graphic<>nil then
    fHighlightTex.Graphic.Transparent:=value;
  if fPressTex.Graphic<>nil then
    fPressTex.Graphic.Transparent:=value;
  Paint;
end;

end.
